#include "scene.h"
#include "v3.h"
#include "M3x3.h"

#include <iostream>


using namespace std;

Scene *scene = 0;

Scene::Scene() {

  gui = new GUI;
  fb = new FrameBuffer(20, 40, 640, 480);
  fb->label("Frame buffer");
  gui->show();
  tmesh = new TMesh();
  tmesh->Load("geometry/teapot1K.bin");
  V3 newCenter(0.0f, 0.0f, -50.0f);
  float newDiag = 30.0f;
  tmesh->PlaceAndScale(newCenter, newDiag);
  Show();
  ppc = new PPC(45.0f, fb->w, fb->h, 10.0f, 100.0f);
  Redraw();

}

void Scene::Show() {

  gui->uiw->position(20+fb->w+20, 40);
  unsigned int color = 0xFF000000;
  fb->Set(color);
  fb->show();

}

void Scene::DBG() {

  {
    V3 newC(50.0f, 25.0f, -50.0f);
    V3 newVD(-1.0f, -.45f, 0.0f);
    V3 quasiUp(0.0f, 1.0f, 0.0f);
    PPC ppc2 = *ppc;
    ppc2.PositionAndOrient(newC, newVD, quasiUp);
    ppc2.Visualize(ppc, fb);
    fb->redraw();
    return;
  }

  {
    V3 newC(50.0f, 25.0f, -50.0f);
    V3 newVD(-1.0f, -.45f, 0.0f);
    V3 quasiUp(0.0f, 1.0f, 0.0f);
    ppc->PositionAndOrient(newC, newVD, quasiUp);
    Redraw();
    return;
  }

  {
    TMesh tmesh;
    tmesh.Load("geometry/teapot1K.bin");
    return;
  }

  {

    M3x3 m;
    m[0] = V3(13.0f, -12.0f, 3.0f);
    m[1] = V3(-4.0f, 3.0f, 2.0f);
    m[2] = V3(5.0f, -12.0f, -10.0f);
    cerr << m << m.Invert() << m.Invert()*m << m*m.Invert();
    return;

  }

  {
    V3 v0(1.0f, 0.3f, 2.0f);
    V3 v1(1.0f, 0.3f, 2.0f);
    V3 v2(0.0f, 0.0f, 0.0f);
    v2 = v0+v1;
    cerr << v2 << endl;
    cerr << v2[0] << endl;
    v2[1] = 34.0f;
    cerr << v2[1] << endl;
    M3x3 m;
    m[0] = v0;
    m[1] = v1;
    m[2] = v2;

  }

  {
    unsigned int colors[2] = {0xFFAAAAAA, 0xFFFFFFFF};
    fb->SetGrid(40, colors);
    fb->redraw();
    return;
  }

  {
    int u, v = 50, sizePix = 15;
    unsigned int color = 0xFFFFFFFF;
    for (u = 0; u < fb->w; u++) {
      fb->Set(0xFF000000);
      fb->DrawDiagonalCross(u, v, color, sizePix);
      fb->redraw();
      Fl::check();
    }
  }

}

void Scene::TranslateForward() {

  float tstep = 10.0f;
  if (!ppc)
    return;
  ppc->Translate(ppc->GetVD()*tstep);
  Redraw();

}
  
void Scene::TranslateBackward() {

  float tstep = 10.0f;
  if (!ppc)
    return;
  ppc->Translate(ppc->GetVD()*-tstep);
  Redraw();

}
  
void Scene::TranslateRight() {

  float tstep = 10.0f;
  if (!ppc)
    return;
  ppc->Translate(ppc->a.UnitVector()*tstep);
  Redraw();

}


void Scene::TranslateLeft() {

  float tstep = 10.0f;
  if (!ppc)
    return;
  ppc->Translate(ppc->a.UnitVector()*-tstep);
  Redraw();

}

void Scene::TranslateUp() {

  float tstep = 10.0f;
  if (!ppc)
    return;
  ppc->Translate(ppc->b.UnitVector()*tstep*-1.0f);
  Redraw();

}

void Scene::TranslateDown() {

  float tstep = 10.0f;
  if (!ppc)
    return;
  ppc->Translate(ppc->b.UnitVector()*tstep);
  Redraw();

}

void Scene::ZoomIn() {

  float frel= 1.1f;
  if (!ppc)
    return;
  ppc->Zoom(frel);
  Redraw();

}

void Scene::Redraw() {

  if (!fb)
    return;
  // clear buffers
  unsigned int bgr = 0xFFFFFFFF;
  fb->Set(bgr);
  fb->SetDepth(0.0f);

  // render geometry
//  tmesh->RenderPoints(ppc, fb);
  tmesh->Render(ppc, fb);

  // post buffer
  fb->redraw();

}
